/************************************************************************************
					 _     _           _     _       _     _   
					| |   (_)         | |   (_)     | |   | |  
					| |    ___   _____| |    _  __ _| |__ | |_ 
					| |   | \ \ / / _ \ |   | |/ _` | '_ \| __|
					| |___| |\ V /  __/ |___| | (_| | | | | |_ 
					\_____/_| \_/ \___\_____/_|\__, |_| |_|\__|
												__/ |          
BETA Firmware 								   |___/
							
EEPROM Algorithms for LiveLight V2 Firmware
Written by Henri Rantanen

Date		6.5.2012
Version		2.5
Target		PCB Version 5, Atmel ATMEGA32-16U

************************************************************************************/
#include "settings.h"

void WriteEE_defaults(void)
{
	cli();	// Disable all interrupts while writing EEPROM	
	
	// Write EEPROM
	eeprom_write_byte ((uint8_t*)	0, 0xDB	);	// Write Check byte
	eeprom_write_byte ((uint8_t*)	1, 96	);	// Write 9600 BAUD rate
	eeprom_write_byte ((uint8_t*)	2, 0x5A	);	// Write Prefix
	eeprom_write_byte ((uint8_t*)	3, 0x20 );	// Write Shut-Down Delay
	eeprom_write_byte ((uint8_t*)	4, 0x01	);	// Write BlackLevel
	eeprom_write_byte ((uint8_t*)	5, 0x01	);	// Write Mode
		
	// Verify EEPROM
	if (~(EE_HEADER == 0xDB && BAUD_RATE == 96 && HEADER_BYTE == 0x5A && SLEEP_DELAY == 0x20 && BLACK_LEVEL == 0x01 && MODE == 0x01))
	{
		// Mark EEPROM as faulty
		eeprom_write_byte ((uint8_t*)0, 0xFF);
		
		//Set back lights to yellow, TC-channel to red
		PORTA = 0x77;	PORTB = 0x0C; PORTC = 0xB5;	PORTD = 0xD8;
	}
}

// Write mode to EEPROM
void SetMode(uint8_t i)
{
	// Mode 0 = Normal Ambilight function, fade out when no input
	// Mode 1 = Store last input if no new data is introduced3
	
	eeprom_write_byte ((uint8_t*)5, i);	// Write to EEPROM
	
	// Set mode by changing the prefix in boblight settings, +1
	// 5A = normal, 5B = with color memory
}

// Read color-table from EEPROM
void ReadEE_Color(void)
{
	volatile uint8_t i;
	
	for(i = 1; i <= Channels+1;i++)
	{
		Compare[i] = RXBuffer[i] = eeprom_read_byte((uint8_t*)i+5); 
	}
}

// Write current color-table to EEPROM
void WriteEE_Color(void)
{
	// Disable all interrupts
	cli();	
	
	volatile uint8_t i;

	for(i = 1; i <= Channels+1; i++)
	{
		// Run 8 PWM cycles to prevent blinking
		for(volatile int j = 0; j < 2040; j++) switch_routine();
		
		// Save one color component
		eeprom_write_byte ((uint8_t*)i+5, RXBuffer[i]);		
	}

	// Re-enable all interrupts
	sei();
}	