/************************************************************************************
					 _     _           _     _       _     _   
					| |   (_)         | |   (_)     | |   | |  
					| |    ___   _____| |    _  __ _| |__ | |_ 
					| |   | \ \ / / _ \ |   | |/ _` | '_ \| __|
					| |___| |\ V /  __/ |___| | (_| | | | | |_ 
					\_____/_| \_/ \___\_____/_|\__, |_| |_|\__|
												__/ |          
BETA Firmware 								   |___/
							
DSP effects for LiveLight V2 Firmware 						 		 ________   __
Written by Henri Rantanen											|  ____\ \ / /
																	| |__   \ V / 
Date		6.5.2012												|  __|   > <  
Version		2.5														| |     / . \ 
Target		PCB Version 5, Atmel ATMEGA32-16U						|_|    /_/ \_\

************************************************************************************/
#include "settings.h"


void ResetShutdownTimer(void)					// Reset time to Off-line
{
	if (TIMSK & ~0x01) TIMSK |= (1 << TOIE0);	// Re-enable shut-down timer after "Off-line"

	TCNT0 = 0;									// Reset Shut-Down timer
}


ISR(TIMER0_OVF_vect)							// TIMER0 Overflow interrupt
{	
	static uint8_t NoInputCounter = 0;			// Off-line timer calculator
	
	if (NoInputCounter > SLEEP_DELAY)			// If counter has reached max. value
	{
		TIMSK = TIMSK & ~(1 << TOIE0);			// Disable TIMER0 interrupt
		NoInputCounter = 0;						// Reset shutdown counter
		
		// Switch mode
		if (RXBuffer[0] == HEADER_BYTE)
		{
			SetMode(0x00);
			FadeOut();			// Fade lights OFF	
		}		
		if (RXBuffer[0] == HEADER_BYTE + 1) 
		{
			SetMode(0x01);
			WriteEE_Color();	// Save colors to EEPROM
		}
	}
	
	else NoInputCounter++;						// Grow Off-line counter
}


void Color(uint8_t Red, uint8_t Green, uint8_t Blue)  // Set RGB color to registers
{
	volatile uint8_t i;

	for(i = 0; i < Channels; i=i+3)				// Browse through all RGB channels
	{
		Compare[i+1] = RXBuffer[i+1] = Red;		// Set channels RED value
		Compare[i+2] = RXBuffer[i+2] = Green;	// Set channels GREEN value
		Compare[i+3] = RXBuffer[i+3] = Blue;	// Set channels BLUE value
	}
}


void FadeOut(void)
{
	volatile uint8_t i;
	volatile uint8_t j;
	
	cli();	// Disable all other functions
	
	// Set saturation to 0%
	for (i = 0; i < 9; i++)
	{		
		RXBuffer[i*3+1] = RXBuffer[i*3+2] = RXBuffer[i*3+3] = ((RXBuffer[i*3+1] + RXBuffer[i*3+2] + RXBuffer[i*3+3]) / 3);
	}

	// Dim all RGB channels
	for (i = 0; i < 255; i++)
	{
		for (j = 0; j<=Channels; j++)
		{
			if (RXBuffer[j] > 0) RXBuffer[j]--;
		}
		
		// Run 8 PWM cycles to prevent blinking
		for(volatile int j = 0; j < 2040; j++) switch_routine();
	}
	
	Color(0,0,0);
	
	sei(); // Enable all interrupts
}