/************************************************************************************
					 _     _           _     _       _     _   
					| |   (_)         | |   (_)     | |   | |  
					| |    ___   _____| |    _  __ _| |__ | |_ 
					| |   | \ \ / / _ \ |   | |/ _` | '_ \| __|
					| |___| |\ V /  __/ |___| | (_| | | | | |_ 
					\_____/_| \_/ \___\_____/_|\__, |_| |_|\__|
												__/ |          
BETA Firmware 								   |___/
							
USART RX Input code for LiveLight V2 Firmware 						 _____  __   __
Written by Henri Rantanen											|  __ \ \ \ / /
																	| |__) | \ V /
Date		6.5.2012												|  _  /   > <  
Version		2.5														| | \ \  / . \ 
Target		PCB Version 5, Atmel ATMEGA32-16U						|_|  \_\/_/ \_\

************************************************************************************/
#include "settings.h"

/* Data input protocol example
	+--------+-------+---------+--------+-------+---------+--------+-------+----------/
	| Header | Red 1 | Green 1 | Blue 1 | Red 2 | Green 2 | Blue 2 | Red 3 | Green3  /
	+--------+-------+---------+--------+-------+---------+--------+-------+--------/
*/

ISR(USART_RXC_vect)							// UART Receive interrupt
{	
	const uint8_t HeaderByte = HEADER_BYTE;	// Read header byte from EEPROM
	const uint8_t BlackLevel = BLACK_LEVEL;	// Read black level from EEPROM
	static volatile uint8_t i = 0;			// Buffer Index
	volatile uint8_t buffer = 0;			// Receiver buffer
	
	// Read USART to buffer
	buffer = UDR;							
	
	// Reset shutdown timer
	ResetShutdownTimer();					
	
	// Check if input is valid or reset index
	if (i > Channels)
	{
		i = 0;
	}
	
	if (RXBuffer[0] != HeaderByte)
	{
		if (RXBuffer[0] != (HeaderByte + 1)) i = 0;
	}
		
	// Read header byte
	if (i == 0)
	{
		RXBuffer[0] = buffer;
	}
	else // Read color data											
	{
		// Apply minimum brightness
		if (buffer <= BlackLevel)
		{
			RXBuffer[i] = BlackLevel;	
		}
		else // Smooth input by calculating average from current and new color
		{
			RXBuffer[i] = Average(RXBuffer[i], buffer);
		}
	}

	// Grow index if header is OK
	if (RXBuffer[0] == HeaderByte || RXBuffer[0] == HeaderByte + 1) i++;		
}


 
uint8_t Average(uint32_t old, uint32_t new)
{		
	// Aggressive attack if change is great enough
	// if (new == 0x00) return new;	// Skip to black
	// if (new == 0xFF) return new;	// Skip to white
	
	// Calculate dynamic moving average (A+B)/2=C via bit shifting (FAST)
	if (new < 0xFF)	new = ((old << 7) + (new << 7)) >> 8;
	if (new < 0x80)	new = ((old << 7) + (new << 7)) >> 8;
	if (new < 0x40)	new = ((old << 7) + (new << 7)) >> 8;
	if (new < 0x20)	new = ((old << 7) + (new << 7)) >> 8;
		 				
	return new;
}