/************************************************************************************
					 _     _           _     _       _     _   
					| |   (_)         | |   (_)     | |   | |  
					| |    ___   _____| |    _  __ _| |__ | |_ 
					| |   | \ \ / / _ \ |   | |/ _` | '_ \| __|
					| |___| |\ V /  __/ |___| | (_| | | | | |_ 
					\_____/_| \_/ \___\_____/_|\__, |_| |_|\__|
												__/ |          
BETA Firmware 								   |___/
							
Settings file for LiveLight V2 Firmware
Written by Henri Rantanen 

Date		6.5.2012
Version		2.5
Target		PCB Version 5, Atmel ATMEGA32-16U

************************************************************************************/

#ifndef SETTINGS_H_
#define SETTINGS_H_

// Module parameters
#define F_CPU 		14745600	// Clock speed in Hz
#define Channels	27			// Amount of channels
												
// Header files
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <util/delay.h>
#include <stdlib.h>

// Global variables
volatile uint8_t Compare  [1 + Channels];		// Stores all the brightness settings in use
volatile uint8_t RXBuffer [1 + Channels];		// USART Receiver buffer

/********* EEPROM ***********
|  Slot	|	Setting			|
|-------|-------------------|
|	0	|	Check byte		|
|	1	|	BAUD_RATE		|
|	2	|	HEADER_BYTE		|
|	3	|	SLEEP_DELAY		|
|	4	|	BLACK_LEVEL		|
|	5	|	MODE			|
|	6	|	CH1 color		|
|	7	|	CH2 color		|
|   --  |   --				|
|	33	|	CH27 color		|
****************************/

#define EE_HEADER		eeprom_read_byte((uint8_t*)0)	// Read EEPROM Header byte
#define	BAUD_RATE		eeprom_read_byte((uint8_t*)1)	// Read BAUD-rate
#define HEADER_BYTE		eeprom_read_byte((uint8_t*)2)	// Read Header byte
#define SLEEP_DELAY		eeprom_read_byte((uint8_t*)3)	// Read SLEEP_DELAY
#define BLACK_LEVEL		eeprom_read_byte((uint8_t*)4)	// Read Minimum brightness
#define MODE			eeprom_read_byte((uint8_t*)5)	// Read current Mode

// Function prototypes
void ResetShutdownTimer(void);							// Resets countdown to off-line mode
void TX_Char(volatile uint8_t data);					// Transmits character to PC
void Initialize(void);									// Initializes the C
void Color(uint8_t Red, uint8_t Green, uint8_t Blue);	// Status Output (RGB)
void FadeOut(void);										// Fade-out all channels
void switch_routine(void);								// PWM switching cycle
void WriteEE_defaults(void);							// Write default settings to EEPROM
uint8_t Average(uint32_t old, uint32_t new);			// Calculate moving average of the input
void SetMode(uint8_t i);								// Save mode
void ReadEE_Color(void);								// Read saved colors from EEPROM
void WriteEE_Color(void);								// Save current colors to EEPROM

#endif /* SETTINGS_H_ */

